from bs4 import BeautifulSoup
import re
import itertools
import xml.etree.ElementTree as ET
from lxml import etree
import lxml.html


def soup(html):
    print('creating soup')
    soup = BeautifulSoup(html, 'lxml')
    print('searching form')
    form = soup.find("form", {'id': 'exportform'})
    if not form:
        return []
    print('searching entries')
    entries = form.find_all("input", {'name': 'item_id'})
    result = [entry['value'] for entry in entries]
    return result


def regex(html):
    r = re.compile(r'<input type="hidden" name="item_id" value="([0-9]+)">')

    s = r.finditer(html)
    result = []
    for ss in itertools.islice(s, 10):
        result.append(ss.group(1))
    return result


def xpath(html):
    root = lxml.html.fromstring(html)
    result = root.xpath('(//div[@class="panel panel-primary"]/div[@class="panel-heading"]/form[@class="form-inline"]/*[@name="item_id"])[position() <= 10]')
    return [r.attrib['value'] for r in result]


def xpath2(html):
    root = etree.HTML(html)
    result = root.xpath('//form[@class="form-inline"]/*[@name="item_id"]')
    return [r.attrib['value'] for r in result]


html = open('/home/turra/example.html').read()

print(xpath(html))

print(xpath2(html))
print(soup(html))
print(regex(html))

import timeit
print(timeit.timeit('soup(html)', setup="from __main__ import soup, html", number=4))
print(timeit.timeit('regex(html)', setup="from __main__ import regex, html", number=4))
print(timeit.timeit('xpath(html)', setup="from __main__ import xpath, html", number=4))
print(timeit.timeit('xpath2(html)', setup="from __main__ import xpath2, html", number=4))



import requests
import bibtexparser
from lxml import etree
from multiprocessing.pool import ThreadPool
from airstuff.workers import OffsetsProducer, CallBackConsumer
#logging.getLogger("requests").setLevel(logging.WARNING)
#logging.getLogger("urllib3").setLevel(logging.WARNING)
#logging.getLogger("bibtexparser").setLevel(logging.WARNING)
import colorlog
logger = colorlog.getLogger('airstuff.air')


def get_document_metadata(document_id):
    BASEURL = "https://air.unimi.it/references"
    r = requests.get(BASEURL, params={'format': 'bibtex', 'item_id': str(document_id)})
    try:
        return bibtexparser.loads(r.text).entries[0]
    except:
        logger.error("problem parsing %s", r.text)
        return None


def get_document_ids_from_author(author_id):
    offset = 0
    BUNCH = 10

    # get = partial(get_document_ids_from_author_offset, author_id=author_id)

    def get(offset):
        return get_document_ids_from_author_offset(author_id, BUNCH, offset)

    while True:
        with ThreadPool(BUNCH) as pool:
            offset_bunch = []
            for b in range(BUNCH):
                offset_bunch.append(offset)
                offset += 20

            r = pool.map(get, offset_bunch)
            for rr in r:
                dobreak = False
                for rrr in rr:
                    yield rrr
                if not rr:
                    dobreak = True
            if dobreak:
                break


def get_document_ids_from_author_offset(author_id, rg, offset):
    BASEURL = "https://air.unimi.it/browse?type=author&order=DESC&rpp=%s&authority=%s&offset=%d"

    url = BASEURL % (rg, author_id, offset)
    logger.debug("getting %s", url)
    r = requests.get(url)
    html = r.text

    root = etree.HTML(html)
    result = root.xpath('//form[@class="form-inline"]/*[@name="item_id"]')
    result = [r.attrib['value'] for r in result]

    logger.debug('results %s', result)
    return result


import threading
import queue


class AirConsumer(threading.Thread):
    def __init__(self, input_queue, output_queue, author_id, step, infos=None, stop_event=None):
        super(AirConsumer, self).__init__()
        self.author_id = author_id
        self.input_queue = input_queue
        self.output_queue = output_queue
        self.step = step
        self.stop_event = stop_event

    def run(self):
        while self.stop_event is None or not self.stop_event.is_set():
            if not self.input_queue.empty():
                offset = self.input_queue.get()
                r = get_document_ids_from_author_offset(self.author_id, self.step, offset)
                for rr in r:
                    info = get_document_metadata(rr)
                    if not info:
                        logging.error('no info in offset %s, step %s', offset, self.step)
                        continue
                    if 'doi' not in info:
                        logger.warning('no doi for %s', info['title'])
                        info['doi'] = None
                    else:
                        info['doi'] = info['doi'].upper()
                    logger.debug('putting info for %s into queue', info['doi'])
                    info['title'] = info['title'].replace('\n', ' ').replace('\t', ' ')
                    info = {k: info[k] for k in ('doi', 'title', 'year')}
                    self.output_queue.put(info)

                self.input_queue.task_done()


class AirQuery():
    def __init__(self, author_id, workers=5, callback=None, buf_size=10):
        self.input_queue = queue.Queue(buf_size)
        self.output_queue = queue.Queue()
        self.author_id = author_id
        self.buf_size = buf_size
        self.all_producers = []
        self.all_workers = []
        self.callback = callback
        self.callback_worker = None
        self.status = 'stopped'
        self.stop_event = threading.Event()
        self.stop_event_callback = threading.Event()
        self.nworkers = workers

    def run(self):
        self.status = 'starting'
        p = OffsetsProducer(self.input_queue, self.buf_size, stop_event=self.stop_event)
        p.setDaemon(True)
        self.all_producers.append(p)

        for w in range(self.nworkers):
            worker = AirConsumer(self.input_queue, self.output_queue, self.author_id, self.buf_size, stop_event=self.stop_event)
            worker.setDaemon(True)
            self.all_workers.append(worker)
            worker.start()

        if self.callback is not None:
            logger.debug('creating callback consumer')
            self.callback_worker = CallBackConsumer(self.output_queue, self.callback, stop_event=self.stop_event_callback)
            self.callback_worker.setDaemon(True)
            self.callback_worker.start()

        p.start()
        self.status = 'running'

    def stop(self):
        self.status = 'stopping'
        self.stop_event.set()

        logger.debug('stopping producer')
        for worker in self.all_producers:
            worker.join()

        logger.debug('stopping consumer')
        for worker in self.all_workers:
            worker.join()

        if self.callback_worker is not None:
            self.stop_event_callback.set()
            logger.debug('stopping callback')
            logger.debug('waiting callback worker to join')
            self.callback_worker.join()

        self.status = 'stopped'
        logger.debug('all stopped')

        self.stop_event.clear()


if __name__ == '__main__':
    import time
    import argparse

    parser = argparse.ArgumentParser(description='Query AIR')
    parser.add_argument('--max-results', type=int, help='stop the query after number of results')
    parser.add_argument('--max-seconds', type=int, help='max number of second for the query')
    parser.add_argument('--out', help='output filename')
    parser.add_argument('--workers', type=int, default=5)
    args = parser.parse_args()

    all_publications = []
    lock = threading.Lock()
    ifound = 0

    fout = None
    if args.out:
        fout = open(args.out, 'w')

    def callback(item):
        global ifound
        global fout
        with lock:
            all_publications.append(item)
            print("%4d %40s %30s %s" % (ifound, item['doi'], str(item['title'][:30]), item['year']))
            if fout is not None:
                fout.write("%s\t%s\t%s\n" % (item['doi'], item['title'], item['year']))
            ifound += 1
    all_publications = []

    lock = threading.Lock()
    ifound = 0

    start_time = time.time()
    q = AirQuery('rp09852', callback=callback, workers=args.workers)
    q.run()
    logger.info("running")
    while True:
        if (args.max_results is not None and ifound >= args.max_results) or \
           (args.max_seconds is not None and (time.time() - start_time) > args.max_seconds):
            logger.info("stopping")
            q.stop()
            logger.info("stopped")
            logger.info("found %d publications" % len(all_publications))
            break

    print("found %d publications" % len(all_publications))

    """
    g = get_document_ids_from_author('rp09852')
    for gg in g:
        info = get_document_metadata(gg)
        if 'doi' not in info:
            logger.info("skipping %s", info)
            continue
        if gg:
            print(info['doi'])
    """


import gi
gi.require_version('Gtk', '3.0')
import colorlog
from gi.repository import Gtk, Gdk
from airstuff.wos import get_wos_from_doi
from airstuff.inspire import query_inspire, fix_info
from airstuff.scopus import get_eid_from_doi
from airstuff import driver_air
from airstuff import journals

logger = colorlog.getLogger('airstuff.info')


class WindowDoi(Gtk.Window):

    def __init__(self, doi=None, institute=None, additional_authors=None):
        Gtk.Window.__init__(self, title="Air Stuff")

        self.info = None
        self.additional_authors = additional_authors

        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        self.browser_name = 'chrome'

        self.set_size_request(1000, 500)
        main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.add(main_box)

        doi_box, _, self.entry_doi = self.create_row("DOI")
        find_button = Gtk.Button.new_with_label("Search")
        find_button.connect("clicked", self.search_doi)
        doi_box.pack_start(find_button, True, True, 0)
        main_box.pack_start(doi_box, True, True, 0)

        institute_box, _, self.entry_institute = self.create_row('institute')
        main_box.pack_start(institute_box, True, True, 0)

        title_box, _, self.entry_title = self.create_row("title", copy=True)
        main_box.pack_start(title_box, True, True, 0)

        selected_authors_box = Gtk.Box()
        selected_authors_box.pack_start(Gtk.Label(label='selected authors'), True, True, 0)
        scrolledwindow = Gtk.ScrolledWindow()
        scrolledwindow.set_hexpand(True)
        scrolledwindow.set_vexpand(True)
        self.selected_authors_textview = Gtk.TextView()
        self.selected_authors_textbuffer = self.selected_authors_textview.get_buffer()
        scrolledwindow.add(self.selected_authors_textview)
        selected_authors_box.pack_start(scrolledwindow, True, True, 0)
        main_box.pack_start(selected_authors_box, True, True, 0)
        button_copy = Gtk.Button.new_with_label('copy')
        button_copy.connect("clicked",
                            lambda w: self.clipboard.set_text(
                                self.selected_authors_textbuffer.get_text(
                                    self.selected_authors_textbuffer.get_start_iter(),
                                    self.selected_authors_textbuffer.get_end_iter(),
                                    True
                                ), -1))
        selected_authors_box.pack_start(button_copy, True, True, 0)

        keywords, _, self.entry_keyworkds = self.create_row('keywords', copy=True)
        main_box.pack_start(keywords, True, True, 0)

        scopus, _, self.entry_scopus = self.create_row('scopus', copy=True)
        main_box.pack_start(scopus, True, True, 0)

        wos, _, self.entry_wos = self.create_row('web of knoledge', copy=True)
        main_box.pack_start(wos, True, True, 0)

        pdf_url, _, self.entry_pdf_url = self.create_row('url link', copy=True)
        main_box.pack_start(pdf_url, True, True, 0)

        frame_selenium = Gtk.Frame(label='automatic insertion')
        main_box.pack_start(frame_selenium, True, True, 8)
        box_selenium = Gtk.Box()
        frame_selenium.add(box_selenium)

        button_chrome = Gtk.RadioButton.new_with_label_from_widget(None, "Chrome")
        button_chrome.connect("toggled", self.on_browser_toggled, "chrome")
        box_selenium.pack_start(button_chrome, False, False, 0)
        button_firefox = Gtk.RadioButton.new_with_label_from_widget(button_chrome, "Firefox")
        button_firefox.connect("toggled", self.on_browser_toggled, "firefox")
        box_selenium.pack_start(button_firefox, False, False, 0)

        button_login_selenium = Gtk.Button(label='login to AIR')
        button_login_selenium.connect('clicked', self.login_selenium)
        box_selenium.pack_start(button_login_selenium, True, True, 0)

        button_start_selenium = Gtk.Button(label='insert from doi')
        button_start_selenium.connect('clicked', self.start_selenium)
        box_selenium.pack_start(button_start_selenium, True, True, 0)

        self.button_pause = Gtk.CheckButton()
        self.button_pause.set_label("Wait after each page")
        box_selenium.pack_start(self.button_pause, True, True, 0)
        self.button_pause.set_active(True)

        if institute is not None:
            self.entry_institute.set_text(institute)

        if doi is not None:
            self.entry_doi.set_text(doi)
            self.search_doi(self)

    def create_row(self, label, copy=False):
        box = Gtk.Box()
        label = Gtk.Label(label=label)
        entry = Gtk.Entry()
        box.pack_start(label, True, True, 0)
        box.pack_start(entry, True, True, 0)

        if copy:
            button_copy = Gtk.Button.new_with_label('copy')
            button_copy.connect("clicked", lambda w: self.clipboard.set_text(entry.get_text(), -1))
            box.pack_start(button_copy, True, True, 0)

        return box, label, entry

    def search_doi(self, widget):
        doi = self.entry_doi.get_text()
        info = query_inspire("doi:%s" % doi)
        if not info == 0:
            pass
        if len(info) > 1:
            pass
        info = self.info = fix_info(info[0])

        self.entry_title.set_text(info['title'])

        all_authors = ';'.join([author['full_name'] for author in info['authors']])

        selected_institutes = set([self.entry_institute.get_text()])
        if not selected_institutes:
            logger.warning('no institute specified')

        selected_authors = [author['full_name'] for author in info['authors']
                            if selected_institutes.intersection(set(author.get('affiliation', [])))]
        self.info['local_authors'] = selected_authors
        if self.additional_authors is not None:
            logger.debug('adding additional authors %s', self.additional_authors)
            for aa in self.additional_authors:
                if aa in self.info['local_authors']:
                    logger.warning('additional author %s already present', aa)
                else:
                    self.info['local_authors'].append(aa)
        if not selected_authors:
            logger.warning('no author found for institute %s', selected_institutes)

        self.selected_authors_textbuffer.set_text('\n'.join(selected_authors))

        eid = get_eid_from_doi(doi)
        if eid is not None:
            self.entry_scopus.set_text(eid)
            info['scopus'] = eid

        wos = get_wos_from_doi(doi)
        if wos is not None:
            info['wos'] = wos
            self.entry_wos.set_text(wos)

        if 'thesaurus_terms' in info:
            keywords = [k['term'] for k in info['thesaurus_terms'] if 'term' in k]
            self.entry_keyworkds.set_text(';'.join(keywords))

        logger.info('getting url from journal')
        pdf_url = journals.get_pdf_url(doi)
        if pdf_url:
            info['pdf_url'] = pdf_url
            self.entry_pdf_url.set_text(pdf_url)

    @property
    def driver(self):
        if hasattr(self, '_driver') and self._driver is not None and self._driver.name == self.browser_name:
            return self._driver
        else:
            self._driver = driver_air.get_driver(debug=True, driver=self.browser_name)
            return self._driver

    def login_selenium(self, widget):
        driver_air.login(self.driver)

    def on_browser_toggled(self, button, value):
        self.browser_name = value

    def start_selenium(self, widget):
        r = driver_air.upload_from_doi(self.driver, self.info, pause=self.button_pause.get_active())
        if r == driver_air.ReturnValue.DUPLICATE:
            logger.warning('do not create duplicate')
            doi = self.info['doi']
            if type(doi) == list:
                doi = doi[0]
            self.ignore_in_future(doi)
        d = self.driver
        d.close()
        del d

    def ignore_in_future(self, doi):
        with open('blacklist.txt', 'a') as f:
            f.write('%s\n' % doi)


def app_main(doi=None, institute=None):
    win = WindowDoi(doi, institute)
    win.connect("destroy", Gtk.main_quit)
    win.show_all()


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(description='Insert information on air')
    parser.add_argument("doi", nargs='?', default=None)
    parser.add_argument('--institute')
    args = parser.parse_args()

    app_main(args.doi, args.institute)
    Gtk.main()



from urllib.parse import urlparse, urljoin
import lxml.html
import requests
import colorlog
from functools import wraps
from time import time
logger = colorlog.getLogger('airstuff.journal')
URL_DOI = 'https://doi.org'


def timing_warning(max_time):
    def wrap(f):
        @wraps(f)
        def wrapped_f(*args, **kwargs):
            tstart = time()
            result = f(*args, **kwargs)
            tstop = time()
            if tstop - tstart > max_time:
                logger.warning('function %s with arguments %s %s took much time: %f ms', f.__name__, args, kwargs, (tstop - tstart) * 1000)
            return result
        return wrapped_f
    return wrap


@timing_warning(1)
def check_url_exists(url):
    return requests.head(url).status_code < 400


@timing_warning(3)
def get_redirect(url):
    r = requests.head(url, allow_redirects=True)
    if r.status_code == 404:
        return None
    location_redirect = r.url
    return location_redirect


def get_redirect_doi(doi):
    url_doi = urljoin(URL_DOI, doi)
    redirect = get_redirect(url_doi)
    if not redirect:
        logger.warning('cannot get redirect for doi %s', url_doi)
        return None
    return redirect


def get_pdf_url(doi):
    logger.debug('getting redirect for doi %s', doi)
    url = get_redirect_doi(doi)
    if not url:
        logger.warning('cannot resolve doi %s', doi)

        tentative_url = None
        if 'epjc' in doi.lower():
            tentative_url = 'http://link.springer.com/content/pdf/%s' % doi
        elif 'jhep' in doi.lower():
            tentative_url = 'https://link.springer.com/content/pdf/%s' % doi

        logger.debug('tentative url: %s from doi: %s', tentative_url, doi)

        if tentative_url:
            if check_url_exists(tentative_url):
                return tentative_url
        
        return None

    hostname = urlparse(url).hostname
    logger.debug('redirect from doi %s is %s', doi, url)
    if hostname == 'link.springer.com':
        return get_pdf_url_springler(doi, url)
    elif hostname == 'journals.aps.org':
        return url.replace('abstract', 'pdf')
    elif hostname == 'linkinghub.elsevier.com':
        return get_pdf_url_science_direct(url)
    elif hostname == 'iopscience.iop.org':
        return urljoin(url, '/pdf')
    else:
        logger.error('not able to get pdf for %s from %s', doi, url)
        return None


def get_pdf_url_science_direct(url):
    if 'linkinghub' in url:
        n = url.split('/')[-1]
        url = 'https://www.sciencedirect.com/science/article/pii/%s/pdfft' % n
        return url
    else:
        raise NotImplementedError


def get_pdf_url_springler(doi, url):
    r = get_pdf_url_springler_tight(url)
    if r:
        return r
    logger.warning('problem to find pdf link for %s on springler, try another method', doi)
    r = get_pdf_url_springler_loose(doi)
    return r

@timing_warning(1)
def get_pdf_url_springler_tight(url):   
    r = requests.get(url)
    html = r.text
    root = lxml.html.fromstring(html)         
    els = root.xpath('//div[@class="download-article test-pdf-link"]//a[@title="Download this article in PDF format"]')

    if len(els) != 1:
        logger.error('more than one download link on %s', url)
        return None
    elif not els:
        logger.error('no download link on %s', url)
        return None

    return urljoin(url, els[0].attrib['href'])


def get_pdf_url_springler_loose(doi):
    url = urljoin('https://link.springer.com/content/pdf/', doi)
    if check_url_exists(url):
        return url
    else:
        return None


# https://labs.inspirehep.net/api/literature?sort=mostrecent&size=25&q=&collaboration=ATLAS
# https://labs.inspirehep.net/api/literature?sort=mostrecent&size=25&q=&collaboration=ATLAS&doc_type=peer%20reviewed
# https://labs.inspirehep.net/api/literature?sort=mostrecent&size=25&q=&collaboration=ATLAS&doc_type=peer%20reviewed&page=2

import requests
import json
from multiprocessing.pool import ThreadPool
import logging
from datetime import datetime
from airstuff.workers import OffsetsProducer, CallBackConsumer, DuplicateFilter
logging.getLogger("requests").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)
logging.getLogger("bibtexparser").setLevel(logging.WARNING)
import colorlog
logger = colorlog.getLogger('airstuff.inspire')


URL_SEARCH = "http://inspirehep.net/search"

ATLAS_QUERY = 'collaboration:"ATLAS" AND collection:published and NOT collection:conferencepaper and collection:citeable'

def query_inspire(query, rg=100, jrec=1, ot=None):
    logging.debug('querying %s, offset=%d', query, jrec)
    # see http://inspirehep.net/help/hacking/search-engine-api
    r = requests.get(URL_SEARCH,
                     params=dict(
                         of='recjson',             # json format
                         rg=rg,                    # range
                         action_search="Search",
                         jrec=jrec,                # offset
                         do='d',
                         ot=ot,                    # ouput tags
                         sf='earliestdate',        # sorting
                         so='d',                   # descending
                         p=query))
    logging.debug('getting %s' % r.url)
    try:
        j = json.loads(r.text)
    except json.decoder.JSONDecodeError:
        logging.error("problem decoding", r.text)
        return None
    return j


def fix_title(title):
    if type(title) is str and title:
        return title
    elif type(title) is str and len(title) == 0:
        return 'NO TITLE'
    elif title is None:
        return 'NO TITLE'
    elif type(title) is dict:
        return title['title']
    elif type(title) is list:
        s = set(t['title'].replace('$', '') for t in title)
        if len(s) == 1:
            title = title[0]['title']
            return title
        else:
            return title[0]['title']  # too complicated (difference is in latex style)
    else:
        logging.warning('cannot parse this title:%s', title)
        return title


def fix_info(info):
    if 'doi' in info and info['doi'] is not None:
        if type(info['doi']) is str:
            info['doi'] = [info['doi']]
        
        info['doi'] = [doi.upper() for doi in info['doi']]
        info['doi'] = sorted(list(set(info['doi'])))
    else:
        info['doi'] = []

    if 'date' in info and type(info['date'] is datetime):
        pass
    else:
        date = '?'
        if 'imprint' in info and info['imprint'] is not None and 'date' in info['imprint']:
            date = info['imprint']['date']
        elif 'prepublication' in info and info['prepublication'] is not None and 'date' in info['prepublication']:
            date = info['prepublication']['date']
        info['date'] = date

    if 'title' in info:
        info['title'] = fix_title(info['title'])    

    return info


def get_all_collaboration(collaboration, infos=None):
    infos = infos or ['recid', 'imprint', 'prepublication', 'number_of_authors', 'system_control_number', 'doi', 'title']
    nthread = 10
    shift = 20
    offset = 1

    def get(offset):
        query = ATLAS_QUERY.replace("ATLAS", collaboration)
        return query_inspire(query, shift, offset, infos)

    while True:
        offset_bunch = []
        for b in range(nthread):
            offset_bunch.append(offset)
            offset += shift
        with ThreadPool(nthread) as pool:
            r = pool.map(get, offset_bunch)
            for rr in r:
                dobreak = False
                for rrr in rr:
                    if rrr['number_of_authors'] is not None and int(rrr['number_of_authors']) < 30:
                        continue
                    yield fix_info(rrr)
                if not rr:
                    dobreak = True
            if dobreak:
                break


import threading
import queue

ndone = 0
nlow_author = 0
lock_ndone = threading.Lock()


class InspireConsumer(threading.Thread):
    def __init__(self, input_queue, output_queue, query, step, infos=None, stop_event=None):
        super(InspireConsumer, self).__init__()
        self.input_queue = input_queue
        self.output_queue = output_queue
        self.query = query
        self.step = step
        self.stop_event = stop_event
        self.infos = infos or ['recid', 'imprint', 'prepublication', 'number_of_authors', 'system_control_number', 'doi', 'title']

    def run(self):
        while self.stop_event is None or not self.stop_event.is_set():
            if not self.input_queue.empty():
                offset = self.input_queue.get()
                r = query_inspire(self.query, self.step, offset, self.infos)

                if len(r) == 0:
                    logging.info("getting empty response")
                    if self.stop_event is not None:
                        self.stop_event.set()
                        logging.debug('stop event set')

                for rr in r:
                    with lock_ndone:
                        global ndone
                        ndone += 1
                    info_fixed = fix_info(rr)
                    if int(info_fixed['number_of_authors']) < 30:  ## TODO: FIXME
                        logging.debug('ignoring %s %s since it has only %d authors',
                                      info_fixed['doi'], info_fixed['title'], info_fixed['number_of_authors'])
                        with lock_ndone:
                            global nlow_author
                            nlow_author += 1
                        continue
                    self.output_queue.put(info_fixed)
                logging.debug('found %d entries from offset %s', len(r), offset)
                self.input_queue.task_done()
        logging.debug('thread at the end')


class ManagerWorker(threading.Thread):
    def __init__(self, stopping_event, stopping_action):
        super(ManagerWorker, self).__init__()
        self.stopping_event = stopping_event
        self.stopping_action = stopping_action

    def run(self):
        self.stopping_event.wait()
        logging.debug('stopping condition met')
        self.stopping_action()
        logging.debug('stopping action done')
        return


class InspireQuery():
    def __init__(self, query, workers=5, callback=None, buf_size=10):
        self.input_queue = queue.Queue(buf_size)
        self.output_queue = queue.Queue()
        self.query = query
        self.buf_size = buf_size
        self.all_producers = []
        self.all_workers = []
        self.callback = callback
        self.callback_worker = None
        self.status = 'stopped'
        self.stop_event = threading.Event()
        self.nworkers = workers

    def run(self):
        self.status = 'starting'
        p = OffsetsProducer(self.input_queue, self.buf_size, stop_event=self.stop_event)
        p.name = 'producer'
        p.setDaemon(True)
        self.all_producers.append(p)

        self.manager = ManagerWorker(stopping_event=self.stop_event,
                                     stopping_action=self.stop)
        self.manager.name = 'manager'
        self.manager.setDaemon(True)
        self.manager.start()
        logging.debug('manager started')

        queue_duplicates = queue.Queue()

        for w in range(self.nworkers):
            worker = InspireConsumer(self.input_queue, self.output_queue, self.query, self.buf_size, stop_event=self.stop_event)
            worker.name = 'consumer-%d' % w
            worker.setDaemon(True)
            self.all_workers.append(worker)
            worker.start()
        logging.debug('worker started')

        #for w in range(2):
        #    worker = DuplicateFilter(queue_duplicates, self.output_queue, stop_event=self.stop_event)
        #    worker.name = 'duplicate-%d' % w
        #    worker.setDaemon(True)
        #    worker.start()
        #logging.debug('worker duplicates started')

        if self.callback is not None:
            self.callback_worker = CallBackConsumer(self.output_queue, self.callback, stop_event=self.stop_event)
            self.callback_worker.name = 'callback'
            self.callback_worker.setDaemon(True)
            self.callback_worker.start()
        logging.debug('callback started')

        p.start()
        logging.debug('produced started')
        self.status = 'running'

    def stop(self):
        logging.debug('start stopping procedure')
        self.status = 'stopping'
        self.stop_event.set()

        logging.info('wait produer to join')
        for worker in self.all_producers:
            worker.join()

        logging.info('wait consumer to join')
        for worker in self.all_workers:
            #worker.join()
            logging.debug('worker %s joined' % worker.name)

        if self.callback_worker is not None:
            logging.info('wait callback to join')
            self.callback_worker.join()

        if threading.get_ident() != self.manager.ident:
            logging.info('wait manager to join')
            self.manager.join()

        self.status = 'stopped'
        logging.info('all stopped')
        logging.info("Number of inspire entries found: %d" % ndone)
        logging.info("Ignored %d entries since low author" % nlow_author)

        self.stop_event.clear()


if __name__ == '__main__':

    import time
    import argparse

    parser = argparse.ArgumentParser(description='Query inspire')
    parser.add_argument('--max-results', type=int, help='stop the query after number of results')
    parser.add_argument('--max-seconds', type=int, help='max number of second for the query')
    parser.add_argument('--out', help='output filename')
    parser.add_argument('--workers', type=int, default=5)
    args = parser.parse_args()

    f = open(args.out, 'w')
    for x in get_all_collaboration('ATLAS'):
        to_write = '%s\t%s\t%s' % (','.join(x['doi']), x['title'], x['date'])
        print(to_write)
        f.write(to_write + '\n')
    exit()


    all_publications = []
    doi_set = set()
    lock = threading.Lock()
    ifound = 0

    fout = None
    if args.out:
        fout = open(args.out, 'w')

    def callback(item):
        global ifound
        global fout
        with lock:
            all_publications.append(item)
            if item['doi'] in doi_set:
                logging.warning('duplicate: %s' % item['doi'])
            doi_set.add(item['doi'])
            print("%4d %40s %30s %s" % (ifound, item['doi'], str(item['title'][:30]), item['imprint']['date']))
            if fout is not None:
                fout.write("%s\t%s\t%s\n" % (item['doi'], item['title'], item['imprint']['date']))
            ifound += 1

    start_time = time.time()
    q = InspireQuery(ATLAS_QUERY, callback=callback, workers=args.workers)
    q.run()
    logging.info("running")
    while True:
        if (args.max_results is not None and ifound >= args.max_results) or \
           (args.max_seconds is not None and (time.time() - start_time) > args.max_seconds) or \
           q.status == 'stopped':
            logging.info("stopping")
            q.stop()
            logging.info("stopped")
            logging.info("found %d publications" % len(all_publications))
            break
    



import tkinter as tk
from tkinter import ttk
import logging
logging.basicConfig(level=logging.DEBUG)

import inspire
from threading import Lock


class Application(tk.Frame):
    def __init__(self, master=None):
        tk.Frame.__init__(self, master)
        self.pack()
        self.create_widgets()

        self.indico_query = None

    def create_widgets(self):
        self.label_input = tk.Label(self, text="inspirehep query:")
        self.label_input.grid(row=0, sticky=tk.W)
        self.input_inspirehep = tk.Entry(self)
        self.input_inspirehep.configure(width=50)
        self.input_inspirehep.grid(row=0, column=1)
        self.input_inspirehep.insert(tk.INSERT, "collaboration:'ATLAS' AND collection:published")
        self.button_run = tk.Button(self, text="run", command=self.run_search_inspire)
        self.button_run.grid(row=0, column=2)
        self.button_stop = tk.Button(self, text="stop", command=self.stop_search_inspire)
        self.button_stop.grid(row=0, column=3)

        cols = 'Date', 'Title', 'doi'
        self.table = ttk.Treeview(self, columns=cols, show='headings')
        # set column headings
        for col in cols:
            self.table.heading(col, text=col)    
        self.table.grid(row=1, column=0, columnspan=3)

        self.table_index = 1

        self.button_login = tk.Button(self, text="login", command=self.air_login)
        self.button_login.grid(row=2, column=0)

        self.button_values = tk.Button(self, text="get values", command=self.air_login)
        self.button_values.grid(row=2, column=1)

        self.button_upload = tk.Button(self, text="upload", command=self.air_login)
        self.button_upload.grid(row=2, column=2)

        self.log = tk.Text(self)
        self.log.grid(row=3, column=0, columnspan=3)

    def run_search_inspire(self):
        self.button_run.config(state="disabled")
        self.master.update()

        lock = Lock()

        def f(item):
            with lock:
                self.update_table(item)
                self.button_stop.update_idletasks()
                self.button_run.update_idletasks()

        #self.indico_query = inspire.IndicoQuery(callback=f)
        #self.indico_query.run()

        self.indico_query = inspire.IndicoQuery()
        self.indico_query.run()

        while self.indico_query.status != 'stopped':
            if not self.indico_query.output_queue.empty():
                item = self.indico_query.output_queue.get()
                self.update_table(item)
                self.button_stop.update_idletasks()
                self.button_run.update_idletasks()

    def update_table(self, item):
        logging.debug('adding to table %s' % item)
        self.table.insert("", self.table_index, self.table_index,
                          values=(item['doi'], item['title']))
        self.table_index += 1

    def stop_search_inspire(self):
        self.indico_query.stop()
        self.indico_query = None
        self.button_run.config(state="enabled")
        self.log.insert(tk.INSERT, "Found %d entries from inspirehep\n" % self.table_index)

    def air_login(self):
        pass


def main():
    root = tk.Tk()
    app = Application(master=root)
    app.master.title("Automate AIR")
    app.master.maxsize(1000, 400)

    app.mainloop()

if __name__ == "__main__":
    main()


import tkinter as tk

import threading
import queue
import time
from threading import Lock


class Producer(threading.Thread):
    def __init__(self, queue):
        super(Producer, self).__init__()
        self.running = True
        self.input_queue = queue

    def run(self):
        i = 0
        while self.running:
            if not self.input_queue.full():
                self.input_queue.put(i)
                i += 1
        return


class Consumer(threading.Thread):
    def __init__(self, input_queue, output_queue):
        super(Consumer, self).__init__()
        self.running = True
        self.input_queue = input_queue
        self.output_queue = output_queue

    def run(self):
        while self.running:
            if not self.input_queue.empty():
                item = self.input_queue.get()
                time.sleep(0.1)
                self.output_queue.put(item ** 2)
                self.input_queue.task_done()


class CallBackConsumer(threading.Thread):
    def __init__(self, input_queue, callback):
        super(CallBackConsumer, self).__init__()
        self.callback = callback
        self.input_queue = input_queue
        self.running = True

    def run(self):
        while True:
            if not self.input_queue.empty():
                item = self.input_queue.get()
                self.callback(item)
                self.input_queue.task_done()
            if not self.running and self.input_queue.empty():
                break


class Manager:
    def __init__(self, callback=None, buf_size=10):
        self.input_queue = queue.Queue(buf_size)
        self.output_queue = queue.Queue()
        self.all_producers = []
        self.all_workers = []
        self.callback = callback
        self.callback_worker = None
        self.status = 'stopped'

    def run(self):
        self.status = 'starting'
        p = Producer(self.input_queue)
        p.setDaemon(True)
        self.all_producers.append(p)

        for w in range(5):
            worker = Consumer(self.input_queue, self.output_queue)
            worker.setDaemon(True)
            self.all_workers.append(worker)
            worker.start()

        if self.callback is not None:
            self.callback_worker = CallBackConsumer(self.output_queue, self.callback)
            self.callback_worker.setDaemon(True)
            self.callback_worker.start()

        p.start()
        self.status = 'running'

    def stop(self):
        self.status = 'stopping'
        print('stopping producer')
        for i in range(len(self.all_producers)):
            self.all_producers[i].running = False
            self.all_producers[i].join()

        print('stopping consumer')
        for i in range(len(self.all_workers)):
            self.all_workers[i].running = False
            self.all_workers[i].join()

        if self.callback_worker is not None:
            self.callback_worker.running = False
            print('waiting callback worker to join')
            self.callback_worker.join()

        self.status = 'stopped'
        print('all stopped')


class Application(tk.Frame):
    def __init__(self, master=None):
        tk.Frame.__init__(self, master)
        self.pack()
        self.create_widgets()
        self.task = None

    def create_widgets(self):
        self.button_run = tk.Button(self, text="run", command=self.run).pack()
        self.button_stop = tk.Button(self, text="stop", command=self.stop).pack()
        self.log = tk.Text(self)
        self.log.pack()

    def run(self):
        lock = Lock()

        def f(item):
            with lock:
                self.debug(item)

        self.task = Manager(callback=f)
        self.task.run()

    def stop(self):
        self.task.stop()
        self.task = None

    def debug(self, item):
        msg = "%d\n" % item
        self.log.insert(tk.INSERT, msg)  # commenting this line it works
        print(msg)


root = tk.Tk()
app = Application(master=root)
app.mainloop()


import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GObject, GLib, Gio
from threading import Lock
import datetime
import logging
import colorlog
from airstuff import inspire
from airstuff.air import AirQuery
from airstuff.air_info import WindowDoi


colors = colorlog.default_log_colors
colors['DEBUG'] = 'blue'
formatter = colorlog.ColoredFormatter('%(log_color)s %(name)s %(levelname)s %(threadName)s %(message)s',
                                      log_colors=colors)
handler = colorlog.StreamHandler()
handler.setFormatter(formatter)
logger = colorlog.getLogger('airstuff')
logger.addHandler(handler)

logger.setLevel(logging.DEBUG)

lock = Lock()


def str2date(date):
    try:
        return datetime.datetime.fromisoformat(date)
    except ValueError:
        try:
            return datetime.datetime.strptime(date, '%Y-%m')
        except ValueError:
            return datetime.datetime.strptime(date, '%Y')
    raise ValueError('cannot parse date %s' % date)


class StatBox(Gtk.Box):
    def __init__(self):
        Gtk.Box.__init__(self)
        self.first_year = 2010
        self.years = ['?', '< %s' % self.first_year]
        self.years += [str(year) for year in range(self.first_year, datetime.date.today().year + 1)]
        self.data = {str(year): 0 for year in self.years}
        self.labels = {}
        self.layout_stat_box()

    def fill(self, year):
        if str(year) not in self.years:
            try:
                y = int(year)
                if y < self.first_year:
                    y = self.years[1]
                else:
                    y = '?'
            except ValueError:
                y = '?'
        else:
            y = str(year)

        self.data[y] += 1
        self.update_labels()

    def set_label(self, year, content):
        self.data[str(year)] = content
        self.update_labels()

    def reset(self):
        for year in self.years:
            self.data[str(year)] = 0
        self.update_labels()

    def update_labels(self):
        for year in self.years:
            self.labels[year].set_text("%s" % self.data[year])

    def layout_stat_box(self):
        for year in self.years:
            label_year = Gtk.Label(label="%s: " % year)
            label_number = Gtk.Label(label='0')
            self.labels[year] = label_number
            self.pack_start(label_year, True, True, 0)
            self.pack_start(label_number, True, True, 0)


class MyWindow(Gtk.Window):

    def __init__(self, air_file=None, inspire_file=None, additional_authors=None):
        Gtk.Window.__init__(self, title="Air Stuff")

        self.set_default_size(800, 350)
        self.create_interface()
        self.additional_authors = additional_authors

        if air_file is not None:
            self.upload_air_from_file(air_file)
        if inspire_file is not None:
            self.upload_inspire_from_file(inspire_file)

        self.file_blacklist = Gio.File.new_for_path(self.entry_blacklist.get_text())
        self.monitor_blacklist = self.file_blacklist.monitor_file(Gio.FileMonitorFlags.NONE, None)
        self.monitor_blacklist.connect("changed", self.changed_file)

    def changed_file(self, m, f, o, event):
        # Without this check, multiple 'ok's will be printed for each file change
        if event == Gio.FileMonitorEvent.CHANGES_DONE_HINT:
            if len(self.table_diff_store):
                logger.debug('redoing table')
                self.make_diff(None)

    def create_interface(self):
        main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.add(main_box)

        # inspire
        box_inspire = Gtk.Box()
        main_box.add(box_inspire)

        # buttons inpire
        self.button_start = Gtk.Button(label="search on inspire")
        self.button_start.connect("clicked", self.run_stop_inspire)
        self.inspire_running = False
        box_inspire.pack_start(self.button_start, True, True, 0)

        self.button_upload_inspire = Gtk.Button(label='upload from file')
        self.button_upload_inspire.connect("clicked", self.upload_inspire)
        box_inspire.pack_start(self.button_upload_inspire, True, True, 0)

        # inspire table
        box_table = Gtk.Box()
        main_box.add(box_table)

        sw = Gtk.ScrolledWindow()
        sw.set_policy(
            Gtk.PolicyType.ALWAYS, Gtk.PolicyType.ALWAYS)
        sw.set_size_request(1000, 150)
        box_table.pack_start(sw, True, True, 0)

        self.table_inspire_store = Gtk.ListStore(str, str, str)
        self.table_view = Gtk.TreeView(model=self.table_inspire_store)
        sw.add(self.table_view)
        self.table_view.set_search_column(1)

        headers = 'doi', 'title', 'date'
        for i in range(3):
            column = Gtk.TreeViewColumn(headers[i], Gtk.CellRendererText(), text=i)
            column.set_sort_column_id(i)
            self.table_view.append_column(column)

        self.inspire_query = None

        # stat box inspire
        self.stat_box_inspire = StatBox()
        main_box.add(self.stat_box_inspire)

        # air button
        box_air = Gtk.Box()
        main_box.add(box_air)

        self.button_start_air = Gtk.Button(label="search on AIR")
        self.button_start_air.connect("clicked", self.run_stop_air)
        self.air_running = False
        box_air.pack_start(self.button_start_air, True, True, 0)

        self.button_upload_air = Gtk.Button(label='upload from file')
        self.button_upload_air.connect("clicked", self.upload_air)
        box_air.pack_start(self.button_upload_air, True, True, 0)

        # air table
        box_table_air = Gtk.Box()
        main_box.add(box_table_air)

        sw = Gtk.ScrolledWindow()
        sw.set_policy(
            Gtk.PolicyType.ALWAYS, Gtk.PolicyType.ALWAYS)
        sw.set_size_request(1000, 150)
        box_table_air.pack_start(sw, True, True, 0)

        self.table_air_store = Gtk.ListStore(str, str, str)
        self.table_air_view = Gtk.TreeView(model=self.table_air_store)
        sw.add(self.table_air_view)
        self.table_air_view.set_search_column(1)

        headers = 'doi', 'title', 'year'
        for i in range(3):
            column = Gtk.TreeViewColumn(headers[i], Gtk.CellRendererText(), text=i)
            column.set_sort_column_id(i)
            self.table_air_view.append_column(column)

        self.stat_box_air = StatBox()
        main_box.add(self.stat_box_air)

        # diff buttons
        box_diff = Gtk.Box()
        button_diff = Gtk.Button(label='make diff')
        button_diff.connect('clicked', self.make_diff)
        box_diff.pack_start(button_diff, True, True, 0)
        main_box.add(box_diff)

        self.entry_blacklist = Gtk.Entry()
        self.entry_blacklist.set_text("blacklist.txt")
        box_diff.pack_start(self.entry_blacklist, True, True, 0)

        self.button_blacklist = Gtk.CheckButton()
        self.button_blacklist.set_label("Remove blacklist")
        box_diff.pack_start(self.button_blacklist, True, True, 0)
        self.button_blacklist.set_active(True)
        self.button_blacklist.connect("toggled", self.remove_blacklist)

        # diff table
        box_table_diff = Gtk.Box()
        main_box.add(box_table_diff)
        sw = Gtk.ScrolledWindow()
        sw.set_policy(
            Gtk.PolicyType.ALWAYS, Gtk.PolicyType.ALWAYS)
        sw.set_size_request(1000, 150)
        box_table_diff.pack_start(sw, True, True, 0)

        self.table_diff_store = Gtk.ListStore(str, str, str)
        self.table_diff_view = Gtk.TreeView(model=self.table_diff_store)
        sw.add(self.table_diff_view)
        self.table_diff_view.set_search_column(1)

        headers = 'doi', 'title', 'date'
        for i in range(3):
            column = Gtk.TreeViewColumn(headers[i], Gtk.CellRendererText(), text=i)
            column.set_sort_column_id(i)
            self.table_diff_view.append_column(column)

        self.stat_box_diff = StatBox()
        main_box.add(self.stat_box_diff)

        # footer
        box_action = Gtk.Box()
        main_box.add(box_action)
        button_go = Gtk.Button(label='process selected')
        button_go.connect('clicked', self.go)
        box_action.pack_start(button_go, True, True, 0)

    def get_blacklist(self):
        fn = self.entry_blacklist.get_text()
        try:
            with open(fn) as f:
                lines = f.read().split('\n')
                return [l for l in lines if l]
        except FileNotFoundError:
            logger.warning('file %s not found', fn)
            return []

    def remove_blacklist(self, button):
        if self.table_diff_store:
            logger.debug('redoing table')
            self.make_diff(None)

    def run_stop_inspire(self, widget):
        if self.inspire_running:
            self.button_start.get_children()[0].set_label('stopping')
            self.button_start.set_sensitive(False)
            self.stop_inspire()
            self.inspire_running = False
            self.button_start.get_children()[0].set_label('start')
            self.button_start.set_sensitive(True)
        else:
            self.button_start.get_children()[0].set_label('starting')
            self.button_start.set_sensitive(False)
            self.run_inspire()
            self.inspire_running = True
            self.button_start.get_children()[0].set_label('stop')
            self.button_start.set_sensitive(True)

    def run_stop_air(self, widget):
        if self.air_running:
            self.button_start_air.get_children()[0].set_label('stopping')
            self.button_start_air.set_sensitive(False)
            self.stop_air()
            self.air_running = False
            self.button_start_air.get_children()[0].set_label('start')
            self.button_start_air.set_sensitive(True)
        else:
            self.button_start_air.get_children()[0].set_label('starting')
            self.button_start_air.set_sensitive(False)
            self.run_air()
            self.air_running = True
            self.button_start_air.get_children()[0].set_label('stop')
            self.button_start_air.set_sensitive(True)

    def run_inspire(self):

        def f(item):
            with lock:
                GLib.idle_add(self.add_inspire, item)

        self.inspire_query = inspire.InspireQuery(query=inspire.ATLAS_QUERY, callback=f)
        self.inspire_query.run()

    def stop_inspire(self):
        self.inspire_query.stop()
        self.inspire_query = None

    def add_inspire(self, item):
        item = inspire.fix_info(item)
        self.table_inspire_store.append([','.join(item['doi']), str(item['title']), str(item['date'])])
        try:
            date = str2date(item['date'])
            self.stat_box_inspire.fill(date.year)
        except ValueError:
            self.stat_box_inspire.fill('?')


    def upload_inspire(self, item):
        dlg = Gtk.FileChooserDialog(title="Please choose a file",
                                    parent=self,
                                    action=Gtk.FileChooserAction.OPEN)
        dlg.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
        dlg.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)

        answer = dlg.run()
        fn = None
        try:
            if answer == Gtk.ResponseType.OK:
                fn = dlg.get_filename()
        finally:
            dlg.destroy()

        if not fn:
            return
        self.upload_inspire_from_file(fn)

    def upload_inspire_from_file(self, fn):
        with open(fn) as f:
            for line in f:
                row = line.split('\t')
                date = row[2].strip()
                item = {'doi': row[0].strip(), 'title': row[1].strip(), 'date': date}

                self.add_inspire(item)

    def run_air(self):

        def f(item):
            GLib.idle_add(self.add_air, item)

        self.air_query = AirQuery('rp09852', callback=f, workers=10)
        self.air_query.run()

    def stop_air(self):
        self.air_query.stop()
        self.air_query = None

    def add_air(self, item):
        self.table_air_store.append([item['doi'], str(item['title']), str(item['year'])])
        self.stat_box_air.fill(item['year'])

    def upload_air(self, item):
        dlg = Gtk.FileChooserDialog(title="Please choose a file",
                                    parent=self,
                                    action=Gtk.FileChooserAction.OPEN)
        dlg.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
        dlg.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)

        answer = dlg.run()
        fn = None
        try:
            if answer == Gtk.ResponseType.OK:
                fn = dlg.get_filename()
        finally:
            dlg.destroy()

        if not fn:
            return
        self.upload_air_from_file(fn)

    def upload_air_from_file(self, fn):
        with open(fn) as f:
            for iline, line in enumerate(f, 1):
                row = line.split('\t')
                try:
                    item = {'doi': row[0].strip(), 'title': row[1].strip(), 'year': row[2].strip()}
                except IndexError:
                    logger.error('problem parsing %s:%d "%s"', fn, iline, line)
                self.add_air(item)

    def make_diff(self, widget):
        logger.debug('making diff table')
        air_values = [list(row) for row in self.table_air_store]
        inspire_values = [list(row) for row in self.table_inspire_store]

        doi_blacklisted = []
        if self.button_blacklist.get_active():
            doi_blacklisted = self.get_blacklist()

        dois_air = set(v[0] for v in air_values)
        dois_inspire_not_air = set()
        info_diff = []
        for inspire_info in inspire_values:
            doi_inspire = inspire_info[0].split(',')
            found = False
            for d in doi_inspire:
                if d in dois_air:
                    found = True
            if not found:
                dois_inspire_not_air = doi_inspire[0]

                info_diff.append([doi_inspire[0], inspire_info[1], inspire_info[2]])

        self.stat_box_diff.reset()
        self.table_diff_store.clear()
        for item in info_diff:
            if not item[0] in doi_blacklisted:
                self.table_diff_store.append([item[0], str(item[1]), str(item[2])])
                self.stat_box_diff.fill(str2date(item[2]).year)

    def go(self, widget):
        index_selected = self.table_diff_view.get_selection().get_selected_rows()
        if not index_selected[1]:
            return
        doi = self.table_diff_store[index_selected[1][0][0]][0]
        new_window = WindowDoi(doi=doi, institute="Milan U.", additional_authors=self.additional_authors)
        new_window.show_all()


def app_main(args):
    win = MyWindow(air_file=args.air_file,
                   inspire_file=args.inspire_file,
                   additional_authors=args.add_author)
    win.connect("destroy", Gtk.main_quit)
    win.show_all()


def main():
    import argparse

    parser = argparse.ArgumentParser(description='Air Stuff')
    parser.add_argument('--air-file', help='txt file with air entries')
    parser.add_argument('--inspire-file', help='txt file with inspire entries')
    parser.add_argument('--add-author', nargs='*', help='add authors to the list. Use format: "Surname, Name"')
    args = parser.parse_args()

    import threading
    threading.Thread(target=lambda: None).start()
    GObject.threads_init()

    app_main(args)
    Gtk.main()


if __name__ == '__main__':
    main()


import requests
import codecs
import logging
import colorlog
logging.getLogger("requests").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)

logger = colorlog.getLogger('airstuff.scopus')

API_KEY = codecs.encode('n929orso1soo5p7r1p5pp994o6188472', 'rot13')


def get_eid_from_doi(doi):
    r = requests.get('https://api.elsevier.com/content/search/scopus',
                     params={'query': 'doi(%s)' % doi,
                             'apiKey': API_KEY})
    json = r.json()
    entries = json['search-results']['entry']
    
    if len(entries) == 0:
        logger.warning("cannot find scopus from doi %s", doi)
        return None
    if len(entries) == 1 and 'error' in entries[0]:
        logger.warning("cannot find scopus from doi %s", doi)
        return None
    if len(entries) != 1:
        logger.warning('multiple results from doi %s', doi)
        return None
    return entries[0]['eid']


if __name__ == '__main__':
    print(get_eid_from_doi('10.1088/1748-0221/14/06/P06012'))
    print(get_eid_from_doi('10.1016/j.physletb.2011.11.010'))


from selenium import webdriver
from selenium.webdriver.support.ui import Select, WebDriverWait
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.remote.remote_connection import LOGGER as selenium_logger
from selenium.common.exceptions import NoSuchElementException
import pickle
import time
import datetime
import requests
import tempfile
from enum import Enum
from colorama import init as init_colorama
from colorama import Fore, Back, Style
import colorlog
import logging
logging.getLogger("requests").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)
logging.getLogger("bibtexparser").setLevel(logging.WARNING)
selenium_logger.setLevel(logging.WARNING)

logger = colorlog.getLogger('airstuff.driverair')
init_colorama()


URL_LOGIN = 'https://air.unimi.it/au/login'
URL_MYDSPACE = 'https://air.unimi.it/mydspace'
URL_SUBMIT = 'https://air.unimi.it/submit'


class ReturnValue(Enum):
    SUCCESS = 1
    DUPLICATE = 2


def get_driver(debug=False, driver='chrome'):
    WINDOW_SIZE = "1920,1080"

    if driver == 'chrome':
        logger.info('creating chrome')
        chrome_options = webdriver.ChromeOptions()
        # chrome_options.add_argument("--incognito")
        if not debug:
            chrome_options.add_argument("--headless")
            chrome_options.add_argument("--mute-audio")
        chrome_options.add_argument("--window-size=%s" % WINDOW_SIZE)

        driver = webdriver.Chrome(options=chrome_options)
    elif driver == 'firefox':
        logger.info('creating firefox')
        firefox_options = webdriver.FirefoxOptions()
        # cookies do not work in firefox private session
        # firefox_options.add_argument("-private")
        if not debug:
            firefox_profile = webdriver.FirefoxProfile()
            firefox_profile.set_preference("media.volume_scale", "0.0")
            firefox_options.profile = firefox_profile
            firefox_options.headless = True
        firefox_options.add_argument("--width=%s" % WINDOW_SIZE.split(',')[0])
        firefox_options.add_argument("--height=%s" % WINDOW_SIZE.split(',')[1])
        driver = webdriver.Firefox(options=firefox_options)

    return driver


def login(driver):
    """Login to the website"""

    driver.get(URL_LOGIN)
    input(Fore.RED + Back.GREEN + Style.DIM + 'press ENTER where you are logged in' + Style.RESET_ALL)
    save_cookies(driver)
    driver.quit()


def save_cookies(driver):
    cookies = driver.get_cookies()
    logger.info('saving %d cookies', len(cookies))
    pickle.dump(cookies, open("cookies.pkl", "wb"))


def load_cookie(driver):
    cookies = pickle.load(open("cookies.pkl", "rb"))
    if not cookies:
        raise IOError("no cookie found. Have you login?")

    for cookie in cookies:
        driver.add_cookie({'name': cookie['name'], 'value': cookie['value']})
    logger.info('%d cookies have been loaded', len(cookies))


def upload_from_doi(driver, info, pause=True):
    driver.get('https://air.unimi.it')
    try:
        load_cookie(driver)
    except IOError:
        logger.info('no cookies found')

    driver.get(URL_SUBMIT)

    if 'login' in driver.current_url:
        logger.warning("You are not logged in")
        input(Fore.RED + Back.GREEN + Style.DIM + 'press ENTER when you are logged in' + Style.RESET_ALL)
        save_cookies(driver)
        driver.get(URL_SUBMIT)

    logger.debug('you are log in')

    page = Page(driver, pause=False)
    page.close_cookie_banner()

    driver.find_element_by_xpath("//a[contains(text(), 'Ricerca per identificativo')]").click()
    element_doi = driver.find_element_by_id("identifier_doi")
    logger.debug('waiting for element to be visible')
    WebDriverWait(driver, 10).until(EC.visibility_of(element_doi))
    element_doi.clear()
    logger.debug('insert doi %s', info['doi'])
    element_doi.send_keys(info['doi'])
    driver.find_element_by_id("lookup_idenfifiers").click()

    # second page
    logger.debug('waiting for page with results from doi %s', info['doi'])
    WebDriverWait(driver, 20).until(EC.visibility_of_element_located((By.ID, "checkresult0"))).click()
    type_document_selector = driver.find_element_by_id("select-collection0")
    sel = Select(type_document_selector)
    sel.select_by_visible_text("01 - Articolo su periodico")
    logger.debug('ask to import selected records')
    driver.find_element_by_xpath("//button[contains(text(), 'Importa i record selezionati')]").click()

    # third page (licence)
    logger.debug('giving licence')
    driver.find_element_by_name("submit_grant").click()

    # check duplicate
    logger.debug('checking for duplicate box')
    duplicate_box_titles = driver.find_elements_by_id('duplicateboxtitle')

    if duplicate_box_titles:
        box = duplicate_box_titles[0]
        logger.debug('sleeping one second')
        time.sleep(1)  # FIXME: the problem is that the page is slow and this will be visible only if there will be a duplicate, which I don't know.
        logger.debug('sleeping finished')
        if box.is_displayed():
            logger.debug("the duplicate box is displayed")
            logger.warning('Trying to insert duplicate')
            WebDriverWait(driver, 10).until(EC.visibility_of_element_located((By.ID, 'cancelpopup'))).click()
            WebDriverWait(driver, 10).until(EC.visibility_of_element_located((By.NAME, 'submit_remove'))).click()
            return ReturnValue.DUPLICATE

    # warning authors
    logger.info("checking if many authors box")
    try:
        many_author_h4 = driver.find_element_by_xpath("//h4[@class='modal-title' and contains(text(), 'Attenzione')]")
        logger.info('box many author found')
        many_authors_close_button = many_author_h4.find_element_by_xpath("//../..//button[contains(text(), 'Chiudi')]")
        logger.debug('closing many authors button found')
        many_authors_close_button.click()
        logger.debug('closed many author box')
    except NoSuchElementException:
        pass

    page = PageDescrivere2(driver, pause=pause)
    page.wait_form_ready()
    logger.debug('filling page Descrivere 2')
    if not page.get_title():
        logger.debug('set title %s', info['title'])
        page.set_title(info['title'])
    else:
        logger.debug('title already present')

    if not page.get_abstract():
        logger.debug('set abstract "%s"', info['abstract'][0]['summary'])
        page.set_abstract(info['abstract'][0]['summary'])
    else:
        logger.debug('abstract already present')
    if not page.get_keywords():
        keywords = [term["term"] for term in info["thesaurus_terms"] if "term" in term]
        logger.debug('set keywords %s', keywords)
        page.set_keywords(keywords)
    else:
        logger.debug('keywords already present')

    driver.find_element_by_id("widgetContributorEdit_dc_authority_people").click()
    authors_field = driver.find_element_by_id("widgetContributorSplitTextarea_dc_authority_people")
    authors_field.clear()
    authors_field.send_keys('; '.join(info['local_authors']))
    driver.find_element_by_id("widgetContributorParse_dc_authority_people").click()

    page.set_type_contribution()
    page.set_type_referee()
    page.set_type_research()
    page.set_type_publication()

    element_field = driver.find_element_by_id("dc_authority_academicField2000")
    element_field.clear()
    element_field.send_keys("FIS/01")
    WebDriverWait(driver, 5).until(EC.element_to_be_clickable((By.XPATH, '//a[text()="Settore FIS/01 - Fisica Sperimentale"]'))).click()

    driver.find_element_by_xpath('//button[@value="Aggiungi ancora"]').click()

    element_field = driver.find_element_by_id("dc_authority_academicField2000")
    element_field.clear()
    element_field.send_keys("FIS/04")
    WebDriverWait(driver, 5).until(EC.element_to_be_clickable((By.XPATH, '//a[text()="Settore FIS/04 - Fisica Nucleare e Subnucleare"]'))).click()

    page.next_page()

    page3 = PageDescrivere3(driver, pause=pause)

    if 'imprint' in info and info['imprint']['date']:
        date = datetime.datetime.fromisoformat(info['imprint']['date'])
        if page3.get_year():
            if int(page3.get_year()) != date.year:
                logger.warning('year is different %s != %s', page3.get_year(), date.year)
        else:
            logger.debug('setting year %s', date.year)
            page3.set_year(date.year)

        if page3.get_month():
            if (int(page3.get_month)) != date.month:
                logger.warning('month is different %s != %s', page3.get_month(), date.month)
        else:
            logger.debug('setting month %s', date.month)
            page3.set_month(date.month)

        if page3.get_day():
            if (int(page3.get_day()) != date.day):
                logger.warning('day is different %s != %s', page3.get_day(), date.day)
        else:
            logger.debug('setting day %s', date.day)
            page3.set_day(date.day)

    page3.set_pub()
    page3.set_rilevanza()
    page3.next_page()

    # page 4
    driver.find_element_by_name("submit_next").click()

    # page 5
    page5 = PageDescrivere5(driver, pause=pause)

    if page5.get_scopus():
        if page5.get_scopus() != info['scopus']:
            logger.warning("scopus reference are different %s != %s", info['scopus'], page5.get_scopus())
    else:
        logger.info('scopus information not found')
        page5.set_scopus(info['scopus'])

    if page5.get_wos():
        if page5.get_wos() != info['wos']:
            logger.warning("wos reference are different %s != %s", info['wos'], page5.get_wos())
    else:
        logger.debug("wos information not found")
        logger.debug("setting wos to %s", info['wos'])
        page5.set_wos(info['wos'])

    page5.set_open()
    page5.set_url('')  # remove url since the automatic one link to the journal and not to the article
    page5.next_page()

    # page 6
    page6 = PageCarica6(driver, pause=pause)

    if info.get('pdf_url', None):
        logger.debug('downloading pdf from %s', info['pdf_url'])
        header = requests.head(info['pdf_url'], allow_redirects=True)
        if header.status_code >= 400:
            logger.error('cannot download pdf with url %s', info['pdf_url'])
        else:
            content_length = header.headers.get('content-length', None)
            if content_length is not None:
                print(content_length)
                logger.debug('downloading %s KB pdf', float(content_length) / 1024.)
            r = requests.get(info['pdf_url'], stream=True, allow_redirects=True)
            with tempfile.NamedTemporaryFile('wb', suffix='.pdf') as ftemp:
                dl = 0
                for chunk in r.iter_content(chunk_size=1024 * 512):
                    if chunk:
                        dl += len(chunk)
                        ftemp.write(chunk)
                        ftemp.flush()
                        if content_length:
                            percent = '%.1f%%' % (dl / float(content_length) * 100)
                        else:
                            percent = ''
                        print('downloaded %d KB %s' % (dl, percent))

                page6.send_file(ftemp.name)

    page6.sito_docente(True)

    page6.next_page()

    # page 6/bis
    page6 = Page(driver, pause)
    page6.next_page()

    return ReturnValue.SUCCESS


class Page:
    next_name = 'submit_next'

    def __init__(self, driver, pause=True):
        self.driver = driver
        self.pause = pause

    def select_hidden(self, element, value):
        old_class = element.get_attribute('class')
        self.driver.execute_script("arguments[0].setAttribute('class', '')", element)
        sel = Select(element)
        sel.select_by_visible_text(value)
        self.driver.execute_script("arguments[0].setAttribute('class', '%s')" % old_class, element)

    def next_page(self):
        if self.pause:
            input(Fore.RED + Back.GREEN + Style.DIM + 'press ENTER to go to next page' + Style.RESET_ALL)
        self.driver.find_element_by_name(self.next_name).click()

    def close_cookie_banner(self):
        el = self.driver.find_elements_by_xpath('//div[@id="jGrowl"]//div[@class="jGrowl-close"]')
        if not el:
            return
        el = el[0]

        if el.is_enabled():
            logger.debug('closing cookies banner')
            el.click()


class PageDescrivere2(Page):

    def set_title(self, title):
        element_title = self.driver.find_element_by_id("dc_title_id")
        element_title.clear()
        element_title.send_keys(title)

    def wait_form_ready(self):
        logger.debug('waiting title to be clickable')
        WebDriverWait(self.driver, 20).until(EC.element_to_be_clickable((By.ID, "dc_title_id")))

    def get_title(self):
        element_title = self.driver.find_element_by_id("dc_title_id")
        return element_title.text

    def set_abstract(self, abstract):
        Select(self.driver.find_element_by_name('dc_description_qualifier')).select_by_visible_text('Inglese')

        element_abstract = self.driver.find_element_by_name('dc_description_value')
        element_abstract.clear()
        element_abstract.send_keys(abstract)

    def get_abstract(self):
        xpath = r'//label[text()="Abstract"]/..//textarea'
        textareas = self.driver.find_elements_by_xpath(xpath)

        for textarea in textareas:
            WebDriverWait(self.driver, 10).until(EC.visibility_of(textarea))
            text = textarea.text
            if text:
                return text

    def set_keywords(self, keywords):
        k = '; '.join(keywords)
        element_keywords = self.driver.find_element_by_id('dc_subject_keywords_id')
        element_keywords.clear()
        element_keywords.send_keys(k)

    def get_keywords(self):
        element_keywords = self.driver.find_element_by_id('dc_subject_keywords_id')
        return element_keywords.text

    def set_type_contribution(self):
        element_type_contribution = self.driver.find_element_by_xpath('//select[@name="dc_type_contribution"]')
        self.select_hidden(element_type_contribution, 'Articolo')

    def set_type_referee(self):
        element_type_referee = self.driver.find_element_by_xpath('//select[@name="dc_type_referee"]')
        self.select_hidden(element_type_referee, 'Esperti anonimi')

    def set_type_research(self):
        element_type_referee = self.driver.find_element_by_xpath('//select[@name="dc_type_research"]')
        self.select_hidden(element_type_referee, 'Ricerca di base')

    def set_type_publication(self):
        element_type_publication = self.driver.find_element_by_xpath('//select[@name="dc_type_publication"]')
        self.select_hidden(element_type_publication, 'Pubblicazione scientifica')


class PageDescrivere3(Page):

    def get_year(self):
        el = self.driver.find_element_by_name("dc_date_issued_year")
        return el.get_attribute('value')

    def set_year(self, year):
        el = self.driver.find_element_by_name("dc_date_issued_year")
        el.clear()
        el.send_keys(str(year))

    def get_day(self):
        el = self.driver.find_element_by_name("dc_date_issued_day")
        return el.get_attribute('value')

    def set_day(self, day):
        el = self.driver.find_element_by_name("dc_date_issued_day")
        el.clear()
        el.send_keys(str(day))

    def get_month(self):
        el = self.driver.find_element_by_name('dc_date_issued_month')
        val = el.get_attribute('value')
        if val == '-1':
            return None
        else:
            return int(val)

    def set_month(self, month):
        el = self.driver.find_element_by_name('dc_date_issued_month')
        sel = Select(el)
        sel.select_by_value(str(month))

    def set_pub(self):
        el = self.driver.find_element_by_xpath('//select[@name="dc_type_publicationstatus"]')
        self.select_hidden(el, 'Pubblicato')

    def set_rilevanza(self):
        el = self.driver.find_element_by_xpath('//select[@name="dc_type_circulation"]')
        self.select_hidden(el, 'Periodico con rilevanza internazionale')


class PageDescrivere5(Page):

    def get_scopus(self):
        els = self.driver.find_elements_by_xpath('//label[text()="Codice identificativo in banca dati"]/..//Select/option[@selected="selected"]')
        scopus_select = None
        for el in els:
            if el.text == 'Scopus':
                scopus_select = el
                break
        if not scopus_select:
            return None

        scopus_id = el.find_element_by_xpath('../../..//input[@class="form-control"]').get_attribute('value')
        if not scopus_id:
            return None
        return scopus_id

    def set_scopus(self, scopus_id):
        els = self.driver.find_elements_by_xpath('//label[text()="Codice identificativo in banca dati"]/..//Select/option[@selected="selected"]')
        scopus_selected = None
        for el in els:
            if el.text == 'Scopus':
                scopus_selected = el
                break
        if scopus_selected:
            scopus_element = scopus_selected.find_element_by_xpath('../../..//input[@class="form-control"]')
            scopus_element.clear()
            scopus_element.send_keys(scopus_id)
        else:
            el2 = els[0].find_element_by_xpath('../../../../../..//select[@name="dc_identifier_qualifier"]')
            sel = Select(el2)
            sel.select_by_value("scopus")
            el_field = el.find_element_by_xpath('../../../../../..//input[@name="dc_identifier_value"]')
            el_field.clear()
            el_field.send_keys(scopus_id)

    def get_wos(self):
        els = self.driver.find_elements_by_xpath('//label[text()="Codice identificativo in banca dati"]/..//Select/option[@selected="selected"]')
        isi_select = None
        for el in els:
            if el.text == 'ISI':
                isi_select = el
                break
        if not isi_select:
            return None

        isi_id = el.find_element_by_xpath('../../..//input[@class="form-control"]').get_attribute('value')
        if not isi_id:
            return None
        return isi_id

    def set_wos(self, isi_id):
        els = self.driver.find_elements_by_xpath('//label[text()="Codice identificativo in banca dati"]/..//Select/option[@selected="selected"]')
        isi_selected = None
        for el in els:
            if el.text == 'ISI':
                isi_selected = el
                break
        if isi_selected:
            isi_element = isi_selected.find_element_by_xpath('../../..//input[@class="form-control"]')
            isi_element.clear()
            isi_element.send_keys(isi_id)
        else:
            el2 = els[0].find_element_by_xpath('../../../../../..//select[@name="dc_identifier_qualifier"]')
            sel = Select(el2)
            sel.select_by_value("isi")
            el_field = els[0].find_element_by_xpath('../../../../../..//input[@name="dc_identifier_value"]')
            el_field.clear()
            el_field.send_keys(isi_id)

    def set_open(self):
        el = self.driver.find_element_by_xpath('//select[@name="dc_iris_checkpolicy"]')
        self.select_hidden(el, 'Aderisco')

    def set_url(self, url):
        el = self.driver.find_element_by_id("dc_identifier_url")
        el.clear()
        if url:
            el.send_keys(url)


class PageCarica6(Page):
    next_name = "submit_upload"

    def send_file(self, fn):
        el = self.driver.find_element_by_id("tfile")
        self.driver.execute_script('arguments[0].style = ""; arguments[0].style.display = "block"; arguments[0].style.visibility = "visible";', el)
        el.send_keys(fn)

    def sito_docente(self, value):
        el = self.driver.find_element_by_id('sitodoc')
        sel = Select(el)
        sel.select_by_value('true' if value else 'false')


if __name__ == '__main__':
    driver = get_driver(debug=True, driver='chrome')

    #login(driver)
    #upload(driver, {'title': 'my title',
    #                'keywords': ['key1', 'key2'],
    #                'abstract': 'my abstract',
    #                'authors': ['Attilio Andreazza', 'Leonardo Carminati']})
    upload_from_doi(driver, {'doi': '10.1140/epjc/s10052-018-6374-z'})



import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GObject, GLib
import threading
import queue
import logging
import time
logging.basicConfig(level=logging.DEBUG)


class Producer(threading.Thread):
    def __init__(self, input_queue, step, stop_event=None):
        super(Producer, self).__init__()
        self.step = step
        self.input_queue = input_queue
        self.stop_event = stop_event

    def run(self):
        i = 0
        while self.stop_event is None or not self.stop_event.is_set():
            if not self.input_queue.full():
                self.input_queue.put(i)
                i += self.step


class WorkerConsumer(threading.Thread):
    def __init__(self, input_queue, output_queue, stop_event=None):
        super(WorkerConsumer, self).__init__()
        self.input_queue = input_queue
        self.output_queue = output_queue
        self.stop_event = stop_event

    def run(self):
        while self.stop_event is not None and not self.stop_event.is_set():
            #time.sleep(0.001)
            item = self.input_queue.get()
            self.output_queue.put(item ** 0.5)
            self.input_queue.task_done()


class CallBackConsumer(threading.Thread):
    def __init__(self, input_queue, callback, stop_event=None):
        super(CallBackConsumer, self).__init__()
        self.callback = callback
        self.input_queue = input_queue
        self.stop_event = stop_event

    def run(self):
        while True:
            if not self.input_queue.empty():
                item = self.input_queue.get()
                self.callback(item)
                self.input_queue.task_done()
            if self.stop_event is not None and self.stop_event.is_set() and self.input_queue.empty():
                break


class MyWindow(Gtk.Window):

    lock = threading.Lock()

    def __init__(self):
        Gtk.Window.__init__(self, title="Air Stuff")

        main_box = Gtk.Box()
        self.add(main_box)
        self.entry = Gtk.Entry()
        main_box.pack_start(self.entry, True, True, 0)

        start_button = Gtk.Button(label='start')
        stop_button = Gtk.Button(label='stop')
        main_box.pack_start(start_button, True, True, 0)
        main_box.pack_start(stop_button, True, True, 0)

        start_button.connect('clicked', self.start)
        stop_button.connect('clicked', self.stop)

    def start(self, widget):
        logging.debug('start')

        def callback(x):
            with self.lock:
                #self.entry.set_text(str(x))
                GLib.idle_add(self.entry.set_text, str(x))

        self.input_queue = queue.Queue(10)
        self.output_queue = queue.Queue()
        self.stop_event = threading.Event()
        self.producer = Producer(self.input_queue, 2, self.stop_event)
        self.producer.start()
        self.consumer = WorkerConsumer(self.input_queue, self.output_queue, self.stop_event)
        self.callback_worker = CallBackConsumer(self.output_queue, callback, self.stop_event)
        self.callback_worker.start()
        self.consumer.start()

    def stop(self, widget):
        logging.debug('stop')
        self.stop_event.set()
        self.producer.join()
        self.consumer.join()


win = MyWindow()
win.connect("destroy", Gtk.main_quit)
win.show_all()
Gtk.main()



import threading
import logging
import colorlog
logger = colorlog.getLogger('airstuff.driverair')


class OffsetsProducer(threading.Thread):
    def __init__(self, input_queue, step, stop_event=None):
        super(OffsetsProducer, self).__init__()
        self.step = step
        self.input_queue = input_queue
        self.stop_event = stop_event

    def run(self):
        i = 0
        while self.stop_event is None or not self.stop_event.is_set():
            if not self.input_queue.full():
                logging.debug("adding %d", i)
                self.input_queue.put(i)
                logging.debug('added %d', i)
                i += self.step
        logging.debug('producer end')
        return


class DuplicateFilter(threading.Thread):
    added = []
    lock_duplicated = threading.Lock()

    def __init__(self, input_queue, output_queue, stop_event=None):
        super(DuplicateFilter, self).__init__()
        self.input_queue = input_queue
        self.output_queue = output_queue
        self.stop_event = stop_event

    def run(self):
        while self.stop_event is None or not self.stop_event.is_set():
            if not self.input_queue.empty():
                item = self.input_queue.get()
                if item in self.added:
                    logging.warning('duplicate: %s', item)
                    self.input_queue.task_done()
                    continue
                with self.lock_duplicated:
                    self.added.append(item)
                self.input_queue.task_done()
                self.output_queue.put(item)


class CallBackConsumer(threading.Thread):
    def __init__(self, input_queue, callback, stop_event=None):
        super(CallBackConsumer, self).__init__()
        self.callback = callback
        self.input_queue = input_queue
        self.stop_event = stop_event

    def run(self):
        while True:
            if not self.input_queue.empty():
                item = self.input_queue.get()
                self.callback(item)
                self.input_queue.task_done()
            if self.stop_event is not None and self.stop_event.is_set() and self.input_queue.empty():
                logging.debug('breaking main loop in callback worker')
                break

        logging.debug("callback worker end")



import requests
import urllib
import colorlog
logger = colorlog.getLogger('airstuff.wos')


def get_wos_from_doi(doi):
    r = requests.get('http://ws.isiknowledge.com/cps/openurl/service',
                     params={'url_ver': r'Z39.88-2004', 'rft_id': r'info:doi/%s' % doi},
                     allow_redirects=False)
    logger.debug('querying %s', r.url)
    location_redirect = r.headers['Location']
    logger.debug('redirect to %s', location_redirect)
    if 'NoRecord' in location_redirect:
        logger.warning('cannot get wos id')
        return None
    url_parsed = urllib.parse.urlparse(location_redirect)
    return urllib.parse.parse_qs(url_parsed.query)['KeyUT'][0]


if __name__ == '__main__':
    print(get_wos_from_doi('10.1016/j.physletb.2011.11.010'))
